// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _getAirplaneMode() {
    try { return Main.panel.statusArea.aggregateMenu._rfkill; } catch(e) { return null; }
}

const GNOME_AIRPLANEMODE = _getAirplaneMode();

const KEY_SHOW_AIRPLANEMODE = 'show-airplanemode';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

let _prototypeSync;

function _overrideFunction(enabled) {
    if (enabled) {
        _prototypeSync = imports.ui.status.rfkill.Indicator.prototype._sync;
        GNOME_AIRPLANEMODE._sync = function() {
            _prototypeSync.call(this);
            let pmShowAirplaneMode = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_AIRPLANEMODE);
            if (pmShowAirplaneMode == PANELMENU_BUTTON_DEFAULT) {
                this._item.menu._getMenuItems()[0].actor.show();
            } else {
                this._indicator.visible = pmShowAirplaneMode == PANELMENU_BUTTON_SHOW;
                this._item.actor.visible = this._indicator.visible;
                let airplaneMode = (this._proxy || this._manager).airplaneMode;
                if (Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12) || Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16)) {
                    this._item.status.text = airplaneMode ? _("On") : _("Off");
                } else {
                    this._item.label.text = airplaneMode ? _("Airplane Mode On") : _("Not In Use");
                }
                this._item.menu._getMenuItems()[0].actor.visible = airplaneMode;
            }
        };
    } else {
        GNOME_AIRPLANEMODE._sync = _prototypeSync;
        GNOME_AIRPLANEMODE._item.menu._getMenuItems()[0].actor.show();
    }
}

const RfkillIndicator = new Lang.Class({
    Name: 'RfkillIndicator',

    _init: function() {
        _overrideFunction(true);
        this._settings =  Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_AIRPLANEMODE, Lang.bind(this, this._updateVisibility));
        if (Convenience.checkGSV(3, 10)) {
            this._connectProxy = GNOME_AIRPLANEMODE._proxy.connect('g-properties-changed', Lang.bind(this, this._updateVisibility));
        } else {
            this._connectManager = GNOME_AIRPLANEMODE._manager.connect('airplane-mode-changed', Lang.bind(this, this._updateVisibility));
        }
        this._updateVisibility();
    },

    _updateVisibility: function() {
        GNOME_AIRPLANEMODE._sync();
    },

    destroy: function() {
        _overrideFunction(false);
        if (Convenience.checkGSV(3, 10)) {
            GNOME_AIRPLANEMODE._proxy.disconnect(this._connectProxy);
        } else {
            GNOME_AIRPLANEMODE._manager.disconnect(this._connectManager);
        }
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility();
    }
});

function getExtension() {
    return GNOME_AIRPLANEMODE ? new RfkillIndicator() : null;
}
