// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

function _getLocation() {
    try { return Main.panel.statusArea.aggregateMenu._location; } catch(e) { return null; }
}

const GNOME_LOCATION = _getLocation();

const KEY_SHOW_LOCATION = 'show-location';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

let _prototypeSyncIndicator;

function _overrideFunction(enabled) {
    if (enabled) {
        _prototypeSyncIndicator = imports.ui.status.location.Indicator.prototype._syncIndicator;
        GNOME_LOCATION._syncIndicator = function() {
            _prototypeSyncIndicator.call(this);
            let pmShowLocation = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_LOCATION);
            if (pmShowLocation != PANELMENU_BUTTON_DEFAULT) {
                this._indicator.visible = pmShowLocation == PANELMENU_BUTTON_SHOW;
                this._item.actor.visible = this._indicator.visible;
            }
        };
    } else {
        GNOME_LOCATION._syncIndicator = _prototypeSyncIndicator;
    }
}

const LocationIndicator = new Lang.Class({
    Name: 'LocationIndicator',

    _init: function() {
        _overrideFunction(true);
        this._settings =  Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_LOCATION, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
    },

    _updateVisibility: function() {
        GNOME_LOCATION._syncIndicator();
    },

    destroy: function() {
        _overrideFunction(false);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility();
    }
});

function getExtension() {
    return GNOME_LOCATION ? new LocationIndicator() : null;
}
